=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2011-08-08 17:03:51 +0000
+++ bzrlib/config.py	2011-08-09 09:27:46 +0000
@@ -2274,40 +2274,92 @@
     value, in which config files it can be stored, etc (TBC).
     """
 
-    def __init__(self, name, default=None):
+    def __init__(self, name, default=None, help=None):
         self.name = name
         self.default = default
+        self.help = help
 
     def get_default(self):
         return self.default
 
 
-# Options registry
-
-option_registry = registry.Registry()
+class OptionRegistry(registry.Registry):
+    """Register config options by their name.
+
+    This overrides ``registry.Registry`` to simplify registration by acquiring
+    some informations from the option object itself.
+    """
+
+    def register(self, option):
+        """Register a new option to its name.
+
+        :param option: The option to register. Its name is used as the key.
+        """
+        super(OptionRegistry, self).register(option.name, option,
+                                             help=option.help)
+
+    def register_lazy(self, key, module_name, member_name):
+        """Register a new option to be loaded on request.
+
+        :param key: This is the key to use to request the option later. Since
+            the registration is lazy, it should be provided and match the
+            option name.
+
+        :param module_name: The python path to the module. Such as 'os.path'.
+
+        :param member_name: The member of the module to return.  If empty or
+                None, get() will return the module itself.
+        """
+        super(OptionRegistry, self).register_lazy(key,
+                                                  module_name, member_name)
+
+    def get_help(self, key=None):
+        """Get the help text associated with the given key"""
+        option = self.get(key)
+        the_help = option.help
+        if callable(the_help):
+            return the_help(self, key)
+        return the_help
+
+
+
+
+option_registry = OptionRegistry()
 
 
 # Registered options in lexicographical order
 
 option_registry.register(
-    'dirstate.fdatasync', Option('dirstate.fdatasync', default=True),
-    help='Flush dirstate changes onto physical disk?')
-option_registry.register(
-    'default_format', Option('default_format', default='2a'),
-    help='Format used when creating branches.')
-option_registry.register(
-    'editor', Option('editor'),
-    help='The command called to launch an editor to enter a message.')
-option_registry.register(
-    'language', Option('language'),
-    help='Language to translate messages into.')
-option_registry.register(
-    'output_encoding', Option('output_encoding'),
-    help='Unicode encoding for output (terminal encoding if not specified).')
-option_registry.register(
-    'repository.fdatasync',
-    Option('repository.fdatasync', default=True),
-    help='Flush repository changes onto physical disk?')
+    Option('dirstate.fdatasync', default=True,
+           help='''
+Flush dirstate changes onto physical disk?
+
+If true (default), working tree metadata changes are flushed through the
+OS buffers to physical disk.  This is somewhat slower, but means data
+should not be lost if the machine crashes.  See also repository.fdatasync.
+'''))
+option_registry.register(
+    Option('default_format', default='2a',
+           help='Format used when creating branches.'))
+option_registry.register(
+    Option('editor',
+           help='The command called to launch an editor to enter a message.'))
+option_registry.register(
+    Option('language',
+           help='Language to translate messages into.'))
+option_registry.register(
+    Option('output_encoding',
+           help= 'Unicode encoding for output'
+           ' (terminal encoding if not specified).'))
+option_registry.register(
+    Option('repository.fdatasync', default=True,
+           help='''\
+Flush repository changes onto physical disk?
+
+If true (default), repository changes are flushed through the OS buffers
+to physical disk.  This is somewhat slower, but means data should not be
+lost if the machine crashes.  See also dirstate.fdatasync.
+'''))
 
 
 class Section(object):

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2011-07-11 10:53:46 +0000
+++ bzrlib/tests/test_config.py	2011-08-09 09:27:46 +0000
@@ -2219,26 +2219,31 @@
     def setUp(self):
         super(TestOptionRegistry, self).setUp()
         # Always start with an empty registry
-        self.overrideAttr(config, 'option_registry', registry.Registry())
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
         self.registry = config.option_registry
 
     def test_register(self):
         opt = config.Option('foo')
-        self.registry.register('foo', opt)
+        self.registry.register(opt)
         self.assertIs(opt, self.registry.get('foo'))
 
-    lazy_option = config.Option('lazy_foo')
-
-    def test_register_lazy(self):
-        self.registry.register_lazy('foo', self.__module__,
-                                    'TestOptionRegistry.lazy_option')
-        self.assertIs(self.lazy_option, self.registry.get('foo'))
-
     def test_registered_help(self):
-        opt = config.Option('foo')
-        self.registry.register('foo', opt, help='A simple option')
+        opt = config.Option('foo', help='A simple option')
+        self.registry.register(opt)
         self.assertEquals('A simple option', self.registry.get_help('foo'))
 
+    lazy_option = config.Option('lazy_foo', help='Lazy help')
+
+    def test_register_lazy(self):
+        self.registry.register_lazy('lazy_foo', self.__module__,
+                                    'TestOptionRegistry.lazy_option')
+        self.assertIs(self.lazy_option, self.registry.get('lazy_foo'))
+
+    def test_registered_lazy_help(self):
+        self.registry.register_lazy('lazy_foo', self.__module__,
+                                    'TestOptionRegistry.lazy_option')
+        self.assertEquals('Lazy help', self.registry.get_help('lazy_foo'))
+
 
 class TestRegisteredOptions(tests.TestCase):
     """All registered options should verify some constraints."""
@@ -2866,6 +2871,9 @@
     # FIXME: This should be parametrized for all known Stack or dedicated
     # paramerized tests created to avoid bloating -- vila 2011-03-31
 
+    def overrideOptionRegistry(self):
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
+
     def test_single_config_get(self):
         conf = dict(foo='bar')
         conf_stack = config.Stack([conf])
@@ -2874,21 +2882,21 @@
     def test_get_with_registered_default_value(self):
         conf_stack = config.Stack([dict()])
         opt = config.Option('foo', default='bar')
-        self.overrideAttr(config, 'option_registry', registry.Registry())
+        self.overrideOptionRegistry()
         config.option_registry.register('foo', opt)
         self.assertEquals('bar', conf_stack.get('foo'))
 
     def test_get_without_registered_default_value(self):
         conf_stack = config.Stack([dict()])
         opt = config.Option('foo')
-        self.overrideAttr(config, 'option_registry', registry.Registry())
+        self.overrideOptionRegistry()
         config.option_registry.register('foo', opt)
         self.assertEquals(None, conf_stack.get('foo'))
 
     def test_get_without_default_value_for_not_registered(self):
         conf_stack = config.Stack([dict()])
         opt = config.Option('foo')
-        self.overrideAttr(config, 'option_registry', registry.Registry())
+        self.overrideOptionRegistry()
         self.assertEquals(None, conf_stack.get('foo'))
 
     def test_get_first_definition(self):

=== modified file 'doc/developers/configuration.txt'
--- doc/developers/configuration.txt	2011-05-18 08:47:16 +0000
+++ doc/developers/configuration.txt	2011-08-09 09:27:46 +0000
@@ -19,8 +19,9 @@
 * default: the default value that Stack.get() should return if no
   value can be found for the option.
 
-Since plugins should be able to lazily register options, the associated help
-is not part of the object but provided at registration time.
+* help: a doc string describing the option, the first line should be a
+  summary and can be followed by a blank line and a more detailed
+  explanation.
 
 Sections
 --------

=== modified file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	2011-08-06 03:52:34 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2011-08-09 09:27:46 +0000
@@ -105,6 +105,9 @@
   two files in different trees have the same contents.
   (Jelmer Vernooij)
 
+* New registry ``OptionRegistry`` specialized for configuration options.
+  (Vincent Ladeuil)
+
 * Remove ``AtomicFile.closed`` which has been deprecated in bzr 0.10.
   (Vincent Ladeuil)
 

